searchNodes=[{"doc":"LiveProps is a library for managing properties and state within Phoenix LiveViews and Phoenix LiveComponents.FeaturesDeclaratively define props and state, initialize default values, and compute derived values using the LiveProps.Prop.prop/3 and LiveProps.Prop.state/3 macros.Supports required propsSupports automatic re-computation of computed props and stateProps automatically added to module documentation.ExampleInside a LiveView or LiveComponent, you must use LiveProps.LiveView or LiveProps.LiveComponent, respectively. LiveComponents can have state and props, while a LiveView can only have state, so we'll look at an example LiveComponent to demonstrate both.Here's a simple Button component that just has props:defmodule ButtonComponent do use Phoenix.LiveComponent use LiveProps.LiveComponent prop :class, :string, default: &quot;button&quot; prop :text, :string, default: &quot;Click me&quot; prop :on_click, :string, default: &quot;click_button&quot; def render(assigns) do ~L&quot;&quot;&quot; &lt;button class=&quot;&lt;%= @class %&gt;&quot; phx-click=&quot;&lt;%= @on_click %&gt;&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; endIn this example we define three props that will be automatically assigned default values, so you don't have to define your own mount or update callbacks to do it yourself.defmodule MyAppWeb.ThermostatComponent do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_component use Phoenix.LiveComponent use LiveProps.LiveComponent prop :user_id, :integer, required: true prop :temperature, :float, compute: :get_temperature state :mode, :atom, default: :verbose def render(assigns) do ~L&quot;&quot;&quot; &lt;%= case @mode do %&gt; &lt;% :verbose -&gt; %&gt; Current temperature: &lt;%= @temperature %&gt; &lt;% _ -&gt; %&gt; &lt;%= @temperature %&gt; &lt;% end %&gt; &lt;button phx-click=&quot;toggle-mode&quot; phx-target=&quot;&lt;%= @myself %&gt;&quot;&gt;Toggle mode&lt;/button&gt; &quot;&quot;&quot; end def get_temperature(assigns) do Thermostat.get_user_reading(assigns.user_id) end def handle_event(&quot;toggle-mode&quot;, _, %{assigns: assigns} = socket) do new_mode = if assigns.mode == :verbose, do: :compact, else: :verbose {:noreply, set_state(socket, :mode, new_mode)} end endOur component requires a :user_id prop, which it uses to fetch the temperature. Since it is required, an error will be raised if you forget to pass it in.We also have the :temperature prop, which is a computed prop. This will be re-calculated automatically anytime the :user_id prop changes. It is calculated by get_temperature/1 which takes the socket assigns as an argument and returns the value to be assigned. Calculations are run in the order defined so we could add even more computed props which depend on the temperature assign.Lastly, the component has a state called :mode which controls the display. We've given it a default value, which is assigned on mount behind the scenes. We could also add computed states that depend on other states. In the &quot;toggle_mode&quot; handler we use LiveProps.States.set_state/3 to update the :mode. We could have just done a regular Phoenix.LiveView.assign/3 call but using set_state/3 is useful when we want to trigger the re-calculation of other states as well (in this case, there are none).Notice what our component does not have: a Phoenix.LiveComponent.mount/1, Phoenix.LiveComponent.update/2 or Phoenix.LiveComponent.preload/1 callback. LiveProps handles that for you, by injecting lightweight callbacks behind the scenes.You can still define your own callbacks if you need to, and everything should continue to work. In a LiveComponent, any callbacks you define will be run after the the LiveProps callbacks (i.e. defaults and computed values will already be assigned to the socket). The LiveProps.LiveComponent documentation has additional information on component lifecycles.This module is not intended to be used directly but rather by means of LiveProps.LiveView and LiveProps.LiveComponent. Please see docs for those modules for additional information.","ref":"LiveProps.html","title":"LiveProps","type":"module"},{"doc":"When you use LiveProps.LiveComponent in your Phoenix LiveComponent, all of the functionality in LiveProps.Props and LiveProps.States will be imported.Exampledefmodule ButtonComponent do use Phoenix.LiveComponent use LiveProps.LiveComponent prop :class, :string, default: &quot;button&quot; prop :text, :string, default: &quot;Click me&quot; prop :on_click, :string, default: &quot;click_button&quot; def render(assigns) do ~L&quot;&quot;&quot; &lt;button class=&quot;&lt;%= @button %&gt;&quot; phx-click=&quot;&lt;%= @on_click %&gt;&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; endIn this example we define three props that will be automatically assigned default values, so you don't have to define your own mount or update callbacks to do it yourself.More examples can be found in the LiveProps documentation.Component LifecycleA Phoenix LiveComponent defines several callbacks, such as mount/1, preload/1, and update/2. LiveProps injects these callbacks under the hood so you don't have to (but you can if you want).If you do not define your own callbacks, the injected ones will be executed as follows: mount/1 ---&gt; update/2 (default and computed (default and computed props states assigned) merged/assigned)States and props will always be assigned in the order defined.If you do define a mount or update callback, they will be run after the associated callback injected by LiveProps. In other words, in your mount/1 callback, default and calculated states will already be assigned to the socket. In your update/2 callback, default and computed props will have been assigned too.If you define a c.Phoenix.LiveComponent.preload/1 callback, which takes a list of assigns, default and computed props will be available in all assigns.PitfallsIf you try to pass a value to a LiveProps.LiveComponent and it has been declared in that component as a state, it will be ignored. (i.e. will not be assigned to the socket). State is meant only to be set within the socket or using LiveProps.States.send_state/3 from a LiveView or another componenet. (You must use LiveProps.States.send_state/3 rather than Phoenix.LiveView.send_update/2)","ref":"LiveProps.LiveComponent.html","title":"LiveProps.LiveComponent","type":"module"},{"doc":"Use this module inside a Phoenix.LiveView to expose to add state to your LiveView.LiveView Lifecycle with LivePropsLiveProps injects lightweight Phoenix.LiveView.mount/3 and Phoenix.LiveView.handle_info/2 callbacks to help manage state.If you define your own mount, it will be run after defaults states have been assigned but before computed states.The reason we assign computed states last is because they may depend on data from params or session. LiveProps does not handle params and session so you will need to manually assign them in your own mount callback, if needed.Example defmodule ThermostatLive do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_view use Phoenix.LiveView use LiveProps.LiveView state :user_id, :integer state :scale, :atom, default: :fahrenheit state :temperature, :float, compute: :get_temperature def render(assigns) do ~L&quot;&quot;&quot; &lt;div&gt; Current temperature is &lt;%= @temperature %&gt; &lt;/div&gt; &quot;&quot;&quot; end def mount(_, %{&quot;current_user_id&quot; =&gt; user_id}, socket) do # socket.assigns.scale already has a default value {:ok, assign(socket, :user_id, user_id)} end def get_temperature(assigns) do Temperature.get_user_reading(assigns.user_id, assigns.scale) end endFirst we defined a :user_id state. This doesn't really do anything other than serve as documentation, since we assign it manually in the mount callback. Still, depending on your preferences, you may find it helpful to have a list of all assigns in one place.Next we defined the :scale state and gave it a default value. This value will be assigned automatically on mount and will be available in any custom mount you define.Finally we defined the :temperature state, with the options compute: :get_temperature. This means this state will be calculated by the get_temperature/1 function, which takes the current assigns as an argument and returns the value to be assigned.","ref":"LiveProps.LiveView.html","title":"LiveProps.LiveView","type":"module"},{"doc":"Functions for working with props. This wil be made available whenever you use LiveProps.LiveComponent within a Phoenix.LiveComponent.","ref":"LiveProps.Props.html","title":"LiveProps.Props","type":"module"},{"doc":"Define a property with the given name and type. Returns :okThis macro is meant to be called within a LiveComponent only. Types can be any atom and are just for documentation purposes.Options::default - A default value to assign to the prop.:required - boolean. If true, an error will be raised if the prop is not passed to the component.:compute - 1-arity function that takes the socket assigns as an argument and returns the value to be assigned. Can be an atom of the name of a function in your component or a remote function call like &amp;MyModule.compute/1. If you use an atom, the referenced function must be public.:doc - String. Will be added to module documentation.","ref":"LiveProps.Props.html#prop/3","title":"LiveProps.Props.prop/3","type":"macro"},{"doc":"Functions for working with states. These will be imported whenever you use LiveProps.LiveComponent or LiveProps.LiveView","ref":"LiveProps.States.html","title":"LiveProps.States","type":"module"},{"doc":"Assign states of the given kind to the socket. The kind can be one of :all, :defaults, :computed (computed states without the :after_connect option), or :async (computed states defined with after_connect: true)","ref":"LiveProps.States.html#assign_states/2","title":"LiveProps.States.assign_states/2","type":"macro"},{"doc":"A replacement for Phoenix.LiveView.send_update/2. Invalid states will be ignored.","ref":"LiveProps.States.html#send_state/3","title":"LiveProps.States.send_state/3","type":"function"},{"doc":"Same as set_state/3 but with a list or map of assigns.","ref":"LiveProps.States.html#set_state/2","title":"LiveProps.States.set_state/2","type":"macro"},{"doc":"Assign the state to the socket and return the socket. This will also trigger re-calculation of any computed state. If you do not wish to do this, use Phoenix.LiveView.assign/3 instead.If the given state is has not been declared as a state, it will be ignored.","ref":"LiveProps.States.html#set_state/3","title":"LiveProps.States.set_state/3","type":"macro"},{"doc":"Same as set_state/2 but raises if passed an invalid state","ref":"LiveProps.States.html#set_state!/2","title":"LiveProps.States.set_state!/2","type":"macro"},{"doc":"Same as set_state/3 but raises if passed an invalid state","ref":"LiveProps.States.html#set_state!/3","title":"LiveProps.States.set_state!/3","type":"macro"},{"doc":"Define state of given name and type. Returns :ok.Types can be any atom and are just for documentation purposes.Options::default - A default value that will be assigned on mount.:compute - 1-arity function that takes the socket assigns as an argument and returns the value to be assigned. Can be an atom of the name of a function in your component or a remote function call like &amp;MyModule.compute/1. If you use an atom, the referenced function must be public.:after_connect - boolean. Only applies in LiveViews. Setting this to true will cause the initial value to be computed asynchronously after the socket connects.","ref":"LiveProps.States.html#state/3","title":"LiveProps.States.state/3","type":"macro"},{"doc":"FormattingIf you want to prevent the mix format command from inserting parentheses around your prop and state definitions, you must edit your .formatter.exs file to include :live_props under the :import_deps options.Your file might look something like this:[ import_deps: [:ecto, :phoenix, :live_props], inputs: [&quot;*.{ex,exs}&quot;, &quot;priv/*/seeds.exs&quot;, &quot;{config,lib,test}/**/*.{ex,exs}&quot;], subdirectories: [&quot;priv/*/migrations&quot;] ]","ref":"formatting.html","title":"Formatting","type":"extras"}]